@@ -22,6 +22,14 @@
  *
  */
 
+/**
+ * SECTION:nbtk-adjustment
+ * @short_description: A GObject representing an adjustable bounded value
+ *
+ * The #NbtkAdjustment object represents a range of values bounded between a
+ * minimum and maximum, together with step and page increments and a page size.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -282,7 +290,7 @@ nbtk_adjustment_class_init (NbtkAdjustmentClass *klass)
                                    g_param_spec_double ("step-increment",
                                                         "Step Increment",
                                                         "Step increment",
-                                                        -G_MAXDOUBLE,
+                                                        0.0,
                                                         G_MAXDOUBLE,
                                                         0.0,
                                                         NBTK_PARAM_READWRITE |
@@ -292,7 +300,7 @@ nbtk_adjustment_class_init (NbtkAdjustmentClass *klass)
                                    g_param_spec_double ("page-increment",
                                                         "Page Increment",
                                                         "Page increment",
-                                                        -G_MAXDOUBLE,
+                                                        0.0,
                                                         G_MAXDOUBLE,
                                                         0.0,
                                                         NBTK_PARAM_READWRITE |
@@ -302,7 +310,7 @@ nbtk_adjustment_class_init (NbtkAdjustmentClass *klass)
                                    g_param_spec_double ("page-size",
                                                         "Page Size",
                                                         "Page size",
-                                                        -G_MAXDOUBLE,
+                                                        0.0,
                                                         G_MAXDOUBLE,
                                                         0.0,
                                                         NBTK_PARAM_READWRITE |
@@ -568,6 +576,9 @@ nbtk_adjustment_set_values (NbtkAdjustment *adjustment,
   gboolean emit_changed = FALSE;
 
   g_return_if_fail (NBTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (page_size >= 0 && page_size <= G_MAXDOUBLE);
+  g_return_if_fail (step_increment >= 0 && step_increment <= G_MAXDOUBLE);
+  g_return_if_fail (page_increment >= 0 && page_increment <= G_MAXDOUBLE);
 
   priv = adjustment->priv;
 
