@@ -20,12 +20,24 @@
  *
  */
 
+/**
+ * SECTION:nbtk-box-layout
+ * @short_description: a layout container arranging children in a single line
+ *
+ * The #NbtkBoxLayout arranges its children along a single line, where each
+ * child can be allocated either its preferred size or larger if the expand
+ * option is set. If the fill option is set, the actor will be allocated more
+ * than its requested size. If the fill option is not set, but the expand option
+ * is enabled, then the position of the actor within the available space can
+ * be determined by the alignment child property.
+ *
+ */
 
 #include "nbtk-box-layout.h"
 
 #include "nbtk-private.h"
 #include "nbtk-scrollable.h"
-
+#include "nbtk-box-layout-child.h"
 
 
 
@@ -297,6 +309,8 @@ nbtk_box_container_iface_init (ClutterContainerIface *iface)
   iface->lower = nbtk_box_container_lower;
   iface->raise = nbtk_box_container_raise;
   iface->sort_depth_order = nbtk_box_container_sort_depth_order;
+
+  iface->child_meta_type = NBTK_TYPE_BOX_LAYOUT_CHILD;
 }
 
 
@@ -375,6 +389,33 @@ nbtk_box_layout_set_property (GObject *object, guint property_id,
     }
 }
 
+static void
+nbtk_box_layout_dispose (GObject *object)
+{
+  NbtkBoxLayoutPrivate *priv = NBTK_BOX_LAYOUT (object)->priv;
+
+  while (priv->children)
+    {
+      clutter_actor_unparent (CLUTTER_ACTOR (priv->children->data));
+
+      priv->children = g_list_delete_link (priv->children, priv->children);
+    }
+
+  if (priv->hadjustment)
+    {
+      g_object_unref (priv->hadjustment);
+      priv->hadjustment = NULL;
+    }
+
+  if (priv->vadjustment)
+    {
+      g_object_unref (priv->vadjustment);
+      priv->vadjustment = NULL;
+    }
+
+  G_OBJECT_CLASS (nbtk_box_layout_parent_class)->dispose (object);
+}
+
 static void
 nbtk_box_layout_get_preferred_width (ClutterActor *actor,
                                      gfloat        for_height,
@@ -426,6 +467,7 @@ nbtk_box_layout_get_preferred_width (ClutterActor *actor,
 
           if (natural_width_p)
             *natural_width_p += child_nat;
+
         }
     }
 
@@ -510,10 +552,11 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
                           ClutterAllocationFlags flags)
 {
   NbtkBoxLayoutPrivate *priv = NBTK_BOX_LAYOUT (actor)->priv;
-  gfloat avail_width, avail_height;
+  gfloat avail_width, avail_height, pref_width, pref_height;
   NbtkPadding padding = { 0, };
   gfloat position = 0;
   GList *l;
+  gint n_expand_children, extra_space;
 
   CLUTTER_ACTOR_CLASS (nbtk_box_layout_parent_class)->allocate (actor, box,
                                                                 flags);
@@ -529,18 +572,19 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
                - padding.top
                - padding.bottom;
 
+  nbtk_box_layout_get_preferred_height (actor, avail_width, NULL,
+                                        &pref_height);
+  nbtk_box_layout_get_preferred_width (actor, avail_height, NULL,
+                                       &pref_width);
+
   /* update adjustments for scrolling */
   if (priv->vadjustment)
     {
       gdouble prev_value;
-      gfloat height;
-
-      clutter_actor_get_preferred_height (actor, avail_width, NULL,
-                                          &height);
 
       g_object_set (G_OBJECT (priv->vadjustment),
                    "lower", 0.0,
-                   "upper", height,
+                   "upper", pref_height,
                    "page-size", avail_height,
                    "step-increment", avail_height / 6,
                    "page-increment", avail_height,
@@ -553,15 +597,10 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
   if (priv->hadjustment)
     {
       gdouble prev_value;
-      gfloat width;
-
-      /* get preferred width for this height */
-      clutter_actor_get_preferred_width (actor, avail_height, NULL,
-                                         &width);
 
       g_object_set (G_OBJECT (priv->hadjustment),
                    "lower", 0.0,
-                   "upper", width,
+                   "upper", pref_width,
                    "page-size", avail_width,
                    "step-increment", avail_width / 6,
                    "page-increment", avail_width,
@@ -571,6 +610,36 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
       nbtk_adjustment_set_value (priv->hadjustment, prev_value);
     }
 
+  /* count the number of children with expand set to TRUE */
+  n_expand_children = 0;
+  for (l = priv->children; l; l = l->next)
+    {
+      gboolean expand;
+      clutter_container_child_get ((ClutterContainer *) actor,
+                                   (ClutterActor*) l->data,
+                                   "expand", &expand,
+                                   NULL);
+      if (expand)
+          n_expand_children++;
+    }
+
+  if (n_expand_children == 0)
+    {
+      extra_space = 0;
+      n_expand_children = 1;
+    }
+  else
+    {
+      if (priv->is_vertical)
+        extra_space = (avail_height - pref_height) / n_expand_children;
+      else
+        extra_space = (avail_width - pref_width) / n_expand_children;
+
+      /* don't shrink anything */
+      if (extra_space < 0)
+        extra_space = 0;
+    }
+
   if (priv->is_vertical)
     position = padding.top;
   else
@@ -581,14 +650,26 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
   else
     l = priv->children;
 
-  while (l)
+  for (l = (priv->is_pack_start) ? g_list_last (priv->children) : priv->children;
+       l;
+       l = (priv->is_pack_start) ? l->prev : l->next)
     {
       ClutterActor *child = (ClutterActor*) l->data;
       ClutterActorBox child_box;
       gfloat child_nat;
+      gboolean xfill, yfill, expand;
+      NbtkAlign xalign, yalign;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE (child))
-        goto next_child;
+        continue;
+
+      clutter_container_child_get ((ClutterContainer*) actor, child,
+                                   "x-fill", &xfill,
+                                   "y-fill", &yfill,
+                                   "x-align", &xalign,
+                                   "y-align", &yalign,
+                                   "expand", &expand,
+                                   NULL);
 
       if (priv->is_vertical)
         {
@@ -596,12 +677,20 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
                                               NULL, &child_nat);
 
           child_box.y1 = position;
-          child_box.y2 = position + child_nat;
+          if (expand)
+            child_box.y2 = position + child_nat + extra_space;
+          else
+            child_box.y2 = position + child_nat;
           child_box.x1 = padding.left;
           child_box.x2 = avail_width;
+
+          _nbtk_allocate_fill (child, &child_box, xalign, yalign, xfill, yfill);
           clutter_actor_allocate (child, &child_box, flags);
 
-          position += (child_nat + priv->spacing);
+          if (expand)
+            position += (child_nat + priv->spacing + extra_space);
+          else
+            position += (child_nat + priv->spacing);
         }
       else
         {
@@ -610,20 +699,22 @@ nbtk_box_layout_allocate (ClutterActor          *actor,
                                              NULL, &child_nat);
 
           child_box.x1 = position;
-          child_box.x2 = position + child_nat;
+
+          if (expand)
+            child_box.x2 = position + child_nat + extra_space;
+          else
+            child_box.x2 = position + child_nat;
+
           child_box.y1 = padding.top;
-          child_box.y2 = avail_width;
+          child_box.y2 = avail_height;
+          _nbtk_allocate_fill (child, &child_box, xalign, yalign, xfill, yfill);
           clutter_actor_allocate (child, &child_box, flags);
 
-          position += (child_nat + priv->spacing);
+          if (expand)
+            position += (child_nat + priv->spacing + extra_space);
+          else
+            position += (child_nat + priv->spacing);
         }
-
-    next_child:
-
-      if (priv->is_pack_start)
-        l = g_list_previous (l);
-      else
-        l = g_list_next (l);
     }
 }
 
@@ -759,6 +850,7 @@ nbtk_box_layout_class_init (NbtkBoxLayoutClass *klass)
 
   object_class->get_property = nbtk_box_layout_get_property;
   object_class->set_property = nbtk_box_layout_set_property;
+  object_class->dispose = nbtk_box_layout_dispose;
 
   actor_class->allocate = nbtk_box_layout_allocate;
   actor_class->get_preferred_width = nbtk_box_layout_get_preferred_width;
@@ -807,12 +899,27 @@ nbtk_box_layout_init (NbtkBoxLayout *self)
   self->priv = BOX_LAYOUT_PRIVATE (self);
 }
 
+/**
+ * nbtk_box_layout_new:
+ *
+ * Create a new #NbtkBoxLayout.
+ *
+ * Returns: a newly allocated #NbtkBoxLayout
+ */
 NbtkWidget *
 nbtk_box_layout_new (void)
 {
   return g_object_new (NBTK_TYPE_BOX_LAYOUT, NULL);
 }
 
+/**
+ * nbtk_box_layout_set_vertical:
+ * @box: A #NbtkBoxLayout
+ * @vertical: #TRUE if the layout should be vertical
+ *
+ * Set the value of the #NbtkBoxLayout::vertical property
+ *
+ */
 void
 nbtk_box_layout_set_vertical (NbtkBoxLayout *box,
                               gboolean       vertical)
@@ -828,6 +935,14 @@ nbtk_box_layout_set_vertical (NbtkBoxLayout *box,
     }
 }
 
+/**
+ * nbtk_box_layout_get_vertical:
+ * @box: A #NbtkBoxLayout
+ *
+ * Get the value of the #NbtkBoxLayout::vertical property.
+ *
+ * Returns: #TRUE if the layout is vertical
+ */
 gboolean
 nbtk_box_layout_get_vertical (NbtkBoxLayout *box)
 {
@@ -836,6 +951,14 @@ nbtk_box_layout_get_vertical (NbtkBoxLayout *box)
   return box->priv->is_vertical;
 }
 
+/**
+ * nbtk_box_layout_set_pack_start:
+ * @box: A #NbtkBoxLayout
+ * @pack_start: #TRUE if the layout should use pack-start
+ *
+ * Set the value of the #NbtkBoxLayout::pack-start property.
+ *
+ */
 void
 nbtk_box_layout_set_pack_start (NbtkBoxLayout *box,
                                 gboolean       pack_start)
@@ -851,6 +974,14 @@ nbtk_box_layout_set_pack_start (NbtkBoxLayout *box,
     }
 }
 
+/**
+ * nbtk_box_layout_get_pack_start:
+ * @box: A #NbtkBoxLayout
+ *
+ * Get the value of the #NbtkBoxLayout::pack-start property.
+ *
+ * Returns: #TRUE if pack-start is enabled
+ */
 gboolean
 nbtk_box_layout_get_pack_start (NbtkBoxLayout *box)
 {
