@@ -35,6 +35,8 @@ typedef struct _DragContext     DragContext;
 struct _DragContext
 {
   NbtkDraggable *draggable;
+  ClutterActor  *stage;
+  ClutterActor  *actor;
 
   guint threshold;
 
@@ -75,8 +77,7 @@ static gboolean
 draggable_release (DragContext        *context,
                    ClutterButtonEvent *event)
 {
-  ClutterActor *actor = CLUTTER_ACTOR (context->draggable);
-  ClutterActor *stage;
+  ClutterActor *stage, *actor;
   gfloat event_x, event_y;
   gfloat actor_x, actor_y;
   gboolean res;
@@ -89,13 +90,18 @@ draggable_release (DragContext        *context,
   actor_x = 0;
   actor_y = 0;
 
+  if (context->actor && !context->emit_delayed_press)
+    actor = context->actor;
+  else
+    actor = CLUTTER_ACTOR (context->draggable);
+
   res = clutter_actor_transform_stage_point (actor,
                                              event_x, event_y,
                                              &actor_x, &actor_y);
   if (!res)
     return FALSE;
 
-  stage = clutter_actor_get_stage (actor);
+  stage = clutter_actor_get_stage (CLUTTER_ACTOR (context->draggable));
 
   context->last_x = actor_x;
   context->last_y = actor_y;
@@ -106,9 +112,10 @@ draggable_release (DragContext        *context,
                                         G_CALLBACK (on_stage_capture),
                                         context);
 
-  g_signal_emit (context->draggable, draggable_signals[DRAG_END], 0,
-                 context->last_x,
-                 context->last_y);
+  if (!context->emit_delayed_press)
+    g_signal_emit (context->draggable, draggable_signals[DRAG_END], 0,
+                   context->last_x,
+                   context->last_y);
 
   g_object_set_data (G_OBJECT (stage), "nbtk-drag-actor", NULL);
 
@@ -119,11 +126,10 @@ static gboolean
 draggable_motion (DragContext        *context,
                   ClutterMotionEvent *event)
 {
-  NbtkDraggable *draggable = NBTK_DRAGGABLE (context->draggable);
-  ClutterActor *actor = CLUTTER_ACTOR (draggable);
   gfloat event_x, event_y;
   gfloat actor_x, actor_y;
   gfloat delta_x, delta_y;
+  ClutterActor *actor;
   gboolean res;
 
   if (!context->in_drag)
@@ -134,6 +140,11 @@ draggable_motion (DragContext        *context,
   actor_x = 0;
   actor_y = 0;
 
+  if (context->actor && !context->emit_delayed_press)
+    actor = context->actor;
+  else
+    actor = CLUTTER_ACTOR (context->draggable);
+
   res = clutter_actor_transform_stage_point (actor,
                                              event_x, event_y,
                                              &actor_x, &actor_y);
@@ -173,6 +184,7 @@ draggable_motion (DragContext        *context,
                          context->press_button,
                          context->press_modifiers);
 
+          actor = CLUTTER_ACTOR (context->draggable);
           stage = clutter_actor_get_stage (actor);
           g_object_set_data (G_OBJECT (stage), "nbtk-drag-actor", actor);
         }
@@ -196,7 +208,17 @@ on_stage_capture (ClutterActor *stage,
     {
     case CLUTTER_MOTION:
       if (context->in_drag)
-        return draggable_motion (context, (ClutterMotionEvent *) event);
+        {
+          ClutterMotionEvent *mevent = (ClutterMotionEvent *)event;
+
+          /* We can miss release events in the case of grabs, so check that
+           * the button is still down here.
+           */
+          if (!(mevent->modifier_state & CLUTTER_BUTTON1_MASK))
+            return draggable_release (context, (ClutterButtonEvent *) event);
+          else
+            return draggable_motion (context, (ClutterMotionEvent *) event);
+        }
       break;
 
     case CLUTTER_BUTTON_RELEASE:
@@ -248,6 +270,7 @@ on_draggable_press (ClutterActor       *actor,
                 "axis", &context->axis,
                 "containment-type", &context->containment,
                 "containment-area", &context->containment_area,
+                "drag-actor", &context->actor,
                 NULL);
 
   if (context->threshold == 0)
@@ -265,6 +288,7 @@ on_draggable_press (ClutterActor       *actor,
 
   context->in_drag = TRUE;
 
+  context->stage = stage;
   g_signal_connect_after (stage,
                           "captured-event", G_CALLBACK (on_stage_capture),
                           context);
@@ -279,7 +303,23 @@ drag_context_free (gpointer data)
     {
       DragContext *context = data;
 
-      g_object_unref (context->draggable);
+      /* disconnect any signal handlers we may have installed */
+      g_signal_handlers_disconnect_by_func (context->draggable,
+                                            G_CALLBACK (on_draggable_press),
+                                            context);
+      if (context->stage)
+        {
+          g_signal_handlers_disconnect_by_func (context->stage,
+                                                G_CALLBACK (on_stage_capture),
+                                                context);
+          context->stage = NULL;
+        }
+
+      if (context->actor)
+        {
+          g_object_unref (G_OBJECT (context->actor));
+          context->actor = NULL;
+        }
 
       if (context->containment_area)
         g_boxed_free (CLUTTER_TYPE_ACTOR_BOX, context->containment_area);
@@ -295,13 +335,15 @@ drag_context_create (NbtkDraggable *draggable)
 
   context = g_slice_new (DragContext);
 
-  context->draggable = g_object_ref (draggable);
+  context->draggable = draggable;
   context->threshold = 0;
   context->axis = 0;
   context->containment = NBTK_DISABLE_CONTAINMENT;
   context->containment_area = NULL;
   context->in_drag = FALSE;
   context->emit_delayed_press = FALSE;
+  context->stage = NULL;
+  context->actor = NULL;
 
   /* attach the context to the draggable */
   g_object_set_qdata_full (G_OBJECT (draggable), quark_draggable_context,
@@ -355,6 +397,7 @@ nbtk_draggable_real_disable (NbtkDraggable *draggable)
   g_signal_handlers_disconnect_by_func (stage,
                                         G_CALLBACK (on_stage_capture),
                                         context);
+  context->stage = NULL;
 
   g_object_set_qdata (G_OBJECT (draggable), quark_draggable_context, NULL);
 
@@ -418,6 +461,14 @@ nbtk_draggable_base_init (gpointer g_iface)
                                  NBTK_PARAM_READWRITE);
       g_object_interface_install_property (g_iface, pspec);
 
+      pspec = g_param_spec_object ("drag-actor",
+                                   "Drag Actor",
+                                   "An actor to use in place of the "
+                                   "draggable while dragging.",
+                                   CLUTTER_TYPE_ACTOR,
+                                   NBTK_PARAM_READWRITE);
+      g_object_interface_install_property (g_iface, pspec);
+
       draggable_signals[DRAG_BEGIN] =
         g_signal_new (I_("drag-begin"),
                       iface_type,
@@ -594,6 +645,27 @@ nbtk_draggable_get_containment_area (NbtkDraggable *draggable,
   g_boxed_free (CLUTTER_TYPE_ACTOR_BOX, box);
 }
 
+void
+nbtk_draggable_set_drag_actor (NbtkDraggable *draggable,
+                               ClutterActor  *actor)
+{
+  g_return_if_fail (NBTK_IS_DRAGGABLE (draggable));
+
+  g_object_set (G_OBJECT (draggable), "drag-actor", actor, NULL);
+}
+
+ClutterActor *
+nbtk_draggable_get_drag_actor (NbtkDraggable *draggable)
+{
+  ClutterActor *actor = NULL;
+
+  g_return_val_if_fail (NBTK_IS_DRAGGABLE (draggable), NULL);
+
+  g_object_get (G_OBJECT (draggable), "drag-actor", &actor, NULL);
+
+  return actor;
+}
+
 void
 nbtk_draggable_enable (NbtkDraggable *draggable)
 {
