@@ -434,10 +434,10 @@ nbtk_entry_allocate (ClutterActor          *actor,
 }
 
 static void
-nbtk_entry_focus_in (ClutterActor *actor)
+clutter_text_focus_in_cb (ClutterText *text,
+                          ClutterActor *actor)
 {
   NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
-  ClutterText *text = CLUTTER_TEXT (priv->entry);
 
   /* remove the hint if visible */
   if (priv->hint
@@ -450,10 +450,10 @@ nbtk_entry_focus_in (ClutterActor *actor)
 }
 
 static void
-nbtk_entry_focus_out (ClutterActor *actor)
+clutter_text_focus_out_cb (ClutterText  *text,
+                           ClutterActor *actor)
 {
   NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
-  ClutterText *text = CLUTTER_TEXT (priv->entry);
 
   /* add a hint if the entry is empty */
   if (priv->hint && !strcmp (clutter_text_get_text (text), ""))
@@ -468,14 +468,6 @@ nbtk_entry_focus_out (ClutterActor *actor)
   clutter_text_set_cursor_visible (text, FALSE);
 }
 
-static void
-clutter_text_focus_in_cb (ClutterActor *actor,
-                          ClutterActor *entry)
-{
-  /* override the clutter text focus handling */
-  clutter_actor_grab_key_focus (entry);
-}
-
 static void
 nbtk_entry_paint (ClutterActor *actor)
 {
@@ -568,6 +560,10 @@ nbtk_entry_key_press_event (ClutterActor    *actor,
 {
   NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
 
+  /* This is expected to handle events that were emitted for the inner
+     ClutterText. They only reach this function if the ClutterText
+     didn't handle them */
+
   /* paste */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
       && event->keyval == CLUTTER_v)
@@ -621,45 +617,17 @@ nbtk_entry_key_press_event (ClutterActor    *actor,
       return TRUE;
     }
 
-  return clutter_actor_event (priv->entry, (ClutterEvent *) event, FALSE);
+  return FALSE;
 }
 
-static gboolean
-nbtk_entry_key_release_event (ClutterActor    *actor,
-                              ClutterKeyEvent *event)
+static void
+nbtk_entry_key_focus_in (ClutterActor *actor)
 {
   NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
 
-  return clutter_actor_event (priv->entry, (ClutterEvent *) event, FALSE);
-}
-
-static gboolean
-nbtk_entry_button_press_event (ClutterActor       *actor,
-                               ClutterButtonEvent *event)
-{
-  NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
-
-  clutter_actor_grab_key_focus (actor);
-
-  return clutter_actor_event (priv->entry, (ClutterEvent *) event, FALSE);
-}
-
-static gboolean
-nbtk_entry_button_release_event (ClutterActor       *actor,
-                                 ClutterButtonEvent *event)
-{
-  NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
-
-  return clutter_actor_event (priv->entry, (ClutterEvent *) event, FALSE);
-}
-
-static gboolean
-nbtk_entry_motion_event (ClutterActor       *actor,
-                         ClutterMotionEvent *event)
-{
-  NbtkEntryPrivate *priv = NBTK_ENTRY_PRIV (actor);
-
-  return clutter_actor_event (priv->entry, (ClutterEvent *) event, FALSE);
+  /* We never want key focus. The ClutterText should be given first
+     pass for all key events */
+  clutter_actor_grab_key_focus (priv->entry);
 }
 
 static void
@@ -679,19 +647,13 @@ nbtk_entry_class_init (NbtkEntryClass *klass)
   actor_class->get_preferred_width = nbtk_entry_get_preferred_width;
   actor_class->get_preferred_height = nbtk_entry_get_preferred_height;
   actor_class->allocate = nbtk_entry_allocate;
-  actor_class->key_focus_in = nbtk_entry_focus_in;
-  actor_class->key_focus_out = nbtk_entry_focus_out;
   actor_class->paint = nbtk_entry_paint;
   actor_class->pick = nbtk_entry_pick;
   actor_class->map = nbtk_entry_map;
   actor_class->unmap = nbtk_entry_unmap;
 
   actor_class->key_press_event = nbtk_entry_key_press_event;
-  actor_class->key_release_event = nbtk_entry_key_release_event;
-
-  actor_class->button_press_event = nbtk_entry_button_press_event;
-  actor_class->button_release_event = nbtk_entry_button_release_event;
-  actor_class->motion_event = nbtk_entry_motion_event;
+  actor_class->key_focus_in = nbtk_entry_key_focus_in;
 
   pspec = g_param_spec_string ("text",
                                "Text",
@@ -752,6 +714,9 @@ nbtk_entry_init (NbtkEntry *entry)
   g_signal_connect (priv->entry, "key-focus-in",
                     G_CALLBACK (clutter_text_focus_in_cb), entry);
 
+  g_signal_connect (priv->entry, "key-focus-out",
+                    G_CALLBACK (clutter_text_focus_out_cb), entry);
+
   priv->spacing = 6.0f;
 
   clutter_actor_set_parent (priv->entry, CLUTTER_ACTOR (entry));
@@ -941,7 +906,7 @@ _nbtk_entry_set_icon_from_file (NbtkEntry     *entry,
 
 
 
-      *icon = (ClutterActor*) nbtk_texture_cache_get_texture (cache, filename, FALSE);
+      *icon = (ClutterActor*) nbtk_texture_cache_get_texture (cache, filename);
 
       clutter_actor_set_reactive (*icon, TRUE);
       clutter_actor_set_parent (*icon, CLUTTER_ACTOR (entry));
