@@ -641,7 +641,7 @@ nbtk_table_calculate_row_heights (NbtkTable *table,
 
       /* calculate the cell width by including any spanned columns */
       cell_width = 0;
-      for (i = 0; i < col_span; i++)
+      for (i = 0; i < col_span && col + i < priv->n_cols; i++)
         cell_width += (float) (col_widths[col + i]);
 
       if (!meta->x_fill)
@@ -837,6 +837,11 @@ nbtk_table_preferred_allocate (ClutterActor          *self,
        * uninitialised memory. We add the spacing in here too since we only
        * want to add as much spacing as times we successfully span.
        */
+      if (col + col_span > priv->n_cols)
+        g_warning ("NbtkTable: col-span exceeds number of columns");
+      if (row + row_span > priv->n_rows)
+        g_warning ("NbtkTable: row-span exceeds number of rows");
+
       if (col_span > 1)
         {
           for (i = col + 1; i < col + col_span && i < priv->n_cols; i++)
@@ -1037,10 +1042,9 @@ nbtk_table_get_preferred_height (ClutterActor *self,
       row_span = meta->row_span;
 
       cell_width = 0;
-      for (i = 0; i < col_span; i++)
+      for (i = 0; i < col_span && col + i < priv->n_cols; i++)
         cell_width += min_widths[col + i];
 
-
       clutter_actor_get_preferred_height (child,
               (float) cell_width, &min, &pref);
 
@@ -1359,6 +1363,9 @@ nbtk_table_add_actor (NbtkTable    *table,
                       gint          row,
                       gint          column)
 {
+  NbtkTableChild *meta;
+  ClutterContainer *container;
+
   g_return_if_fail (NBTK_IS_TABLE (table));
   g_return_if_fail (CLUTTER_IS_ACTOR (actor));
   g_return_if_fail (row >= -1);
@@ -1370,12 +1377,15 @@ nbtk_table_add_actor (NbtkTable    *table,
   if (column < 0)
     column = table->priv->n_cols + 1;
 
-  clutter_container_add_actor (CLUTTER_CONTAINER (table), actor);
-  clutter_container_child_set (CLUTTER_CONTAINER (table), actor,
-                               "row", row,
-                               "col", column,
-                               NULL);
+  container = CLUTTER_CONTAINER (table);
+  clutter_container_add_actor (container, actor);
 
+  meta = (NbtkTableChild *) clutter_container_get_child_meta (container, actor);
+  meta->row = row;
+  meta->col = column;
+  _nbtk_table_update_row_col (table, row, column);
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 }
 
 /**
@@ -1416,16 +1426,17 @@ nbtk_table_add_actor_with_properties (NbtkTable    *table,
 
   container = (ClutterContainer *)table;
   clutter_container_add_actor (container, actor);
-  clutter_container_child_set (CLUTTER_CONTAINER (table), actor,
-                               "row", row,
-                               "col", column,
-                               NULL);
-  meta = (NbtkTableChild*) clutter_container_get_child_meta (container,
-                                                             actor);
+
+  meta = (NbtkTableChild *) clutter_container_get_child_meta (container, actor);
+  meta->row = row;
+  meta->col = column;
+  _nbtk_table_update_row_col (table, row, column);
 
   va_start (args, first_property_name);
   g_object_set_valist ((GObject*) meta, first_property_name, args);
   va_end (args);
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 }
 
 /**
