@@ -23,6 +23,18 @@
  *
  */
 
+/**
+ * SECTION:nbtk-viewport
+ * @short_description: single child scrollable container
+ *
+ * #NbtkViewport allows non-scrollable children to be scrollable by implementing
+ * the #NbtkScrollable and #ClutterContainer interface.
+ *
+ * Do not use #NbtkViewport if you need good performance as it does can not
+ * be selective about the area of its child that is painted/picked. Therefore
+ * if the child is very large or contains a lot of children, you will experience
+ * poor performance.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -91,15 +103,15 @@ nbtk_viewport_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_X_ORIGIN:
-      g_value_set_int (value, (int) priv->x);
+      g_value_set_float (value, priv->x);
       break;
 
     case PROP_Y_ORIGIN:
-      g_value_set_int (value, (int) priv->y);
+      g_value_set_float (value, priv->y);
       break;
 
     case PROP_Z_ORIGIN:
-      g_value_set_int (value, (int) priv->z);
+      g_value_set_float (value, priv->z);
       break;
 
     case PROP_HADJUST :
@@ -134,24 +146,24 @@ nbtk_viewport_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_X_ORIGIN:
-      nbtk_viewport_set_originu (viewport,
-                                 g_value_get_int (value),
-                                 priv->y,
-                                 priv->z);
+      nbtk_viewport_set_origin (viewport,
+                                g_value_get_float (value),
+                                priv->y,
+                                priv->z);
       break;
 
     case PROP_Y_ORIGIN:
-      nbtk_viewport_set_originu (viewport,
-                                 priv->x,
-                                 g_value_get_int (value),
-                                 priv->z);
+      nbtk_viewport_set_origin (viewport,
+                                priv->x,
+                                g_value_get_float (value),
+                                priv->z);
       break;
 
     case PROP_Z_ORIGIN:
-      nbtk_viewport_set_originu (viewport,
-                                 priv->x,
-                                 priv->y,
-                                 g_value_get_int (value));
+      nbtk_viewport_set_origin (viewport,
+                                priv->x,
+                                priv->y,
+                                g_value_get_float (value));
       break;
 
     case PROP_HADJUST :
@@ -299,7 +311,8 @@ nbtk_viewport_allocate (ClutterActor          *self,
         {
           g_object_set (G_OBJECT (priv->hadjustment),
                        "lower", 0.0,
-                       "upper", MAX (0, natural_width - available_width),
+                       "page-size", available_width,
+                       "upper", natural_width,
                        NULL);
 
           /* Make sure value is clamped */
@@ -311,7 +324,8 @@ nbtk_viewport_allocate (ClutterActor          *self,
         {
           g_object_set (G_OBJECT (priv->vadjustment),
                        "lower", 0.0,
-                       "upper", MAX (0, natural_height - available_height),
+                       "page-size", available_height,
+                       "upper", natural_height,
                        NULL);
 
           prev_value = nbtk_adjustment_get_value (priv->vadjustment);
@@ -323,6 +337,7 @@ nbtk_viewport_allocate (ClutterActor          *self,
 static void
 nbtk_viewport_class_init (NbtkViewportClass *klass)
 {
+  GParamSpec *pspec;
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
@@ -336,52 +351,39 @@ nbtk_viewport_class_init (NbtkViewportClass *klass)
   actor_class->pick = nbtk_viewport_pick;
   actor_class->allocate = nbtk_viewport_allocate;
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_X_ORIGIN,
-                                   g_param_spec_int ("x-origin",
-                                                     "X Origin",
-                                                     "Origin's X coordinate in pixels",
-                                                     -G_MAXINT, G_MAXINT,
-                                                     0,
-                                                     G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_Y_ORIGIN,
-                                   g_param_spec_int ("y-origin",
-                                                     "Y Origin",
-                                                     "Origin's Y coordinate in pixels",
-                                                     -G_MAXINT, G_MAXINT,
-                                                     0,
-                                                     G_PARAM_READWRITE));
+  pspec = g_param_spec_float ("x-origin",
+                              "X Origin",
+                              "Origin's X coordinate in pixels",
+                              -G_MAXFLOAT, G_MAXFLOAT, 0,
+                              NBTK_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_X_ORIGIN, pspec);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_Z_ORIGIN,
-                                   g_param_spec_int ("z-origin",
-                                                     "Z Origin",
-                                                     "Origin's Z coordinate in pixels",
-                                                     -G_MAXINT, G_MAXINT,
-                                                     0,
-                                                     G_PARAM_READWRITE));
+  pspec = g_param_spec_float ("y-origin",
+                              "Y Origin",
+                              "Origin's Y coordinate in pixels",
+                              -G_MAXFLOAT, G_MAXFLOAT, 0,
+                              NBTK_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_Y_ORIGIN, pspec);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_SYNC_ADJUST,
-                                   g_param_spec_boolean ("sync-adjustments",
-                                                         "Synchronise "
-                                                         "adjustments",
-                                                         "Whether to "
-                                                         "synchronise "
-                                                         "adjustments with "
-                                                         "viewport size",
-                                                         TRUE,
-                                                         G_PARAM_READWRITE));
+  pspec = g_param_spec_float ("z-origin",
+                              "Z Origin",
+                              "Origin's Z coordinate in pixels",
+                              -G_MAXFLOAT, G_MAXFLOAT, 0,
+                              NBTK_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_Z_ORIGIN, pspec);
 
-  g_object_class_override_property (gobject_class,
-                                    PROP_HADJUST,
-                                    "hadjustment");
+  pspec = g_param_spec_boolean ("sync-adjustments",
+                                "Synchronise adjustments",
+                                "Whether to synchronise adjustments with "
+                                "viewport size",
+                                TRUE,
+                                NBTK_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_SYNC_ADJUST, pspec);
 
-  g_object_class_override_property (gobject_class,
-                                    PROP_VADJUST,
-                                    "vadjustment");
+  g_object_class_override_property (gobject_class, PROP_HADJUST, "hadjustment");
+
+  g_object_class_override_property (gobject_class, PROP_VADJUST, "vadjustment");
 }
 
 static void
@@ -394,10 +396,10 @@ hadjustment_value_notify_cb (NbtkAdjustment *adjustment,
 
   value = nbtk_adjustment_get_value (adjustment);
 
-  nbtk_viewport_set_originu (viewport,
-                             (float) (value),
-                             priv->y,
-                             priv->z);
+  nbtk_viewport_set_origin (viewport,
+                            (float) (value),
+                            priv->y,
+                            priv->z);
 }
 
 static void
@@ -409,10 +411,10 @@ vadjustment_value_notify_cb (NbtkAdjustment *adjustment, GParamSpec *arg1,
 
   value = nbtk_adjustment_get_value (adjustment);
 
-  nbtk_viewport_set_originu (viewport,
-                             priv->x,
-                             (float) (value),
-                             priv->z);
+  nbtk_viewport_set_origin (viewport,
+                            priv->x,
+                            (float) (value),
+                            priv->z);
 }
 
 static void
@@ -546,35 +548,6 @@ scrollable_interface_init (NbtkScrollableInterface *iface)
   iface->get_adjustments = scrollable_get_adjustments;
 }
 
-static void
-clip_notify_cb (ClutterActor *actor,
-                GParamSpec   *pspec,
-                NbtkViewport *self)
-{
-  NbtkViewportPrivate *priv = self->priv;
-  gfloat width, height;
-
-  if (!priv->sync_adjustments)
-    return;
-
-  if (!clutter_actor_has_clip (actor))
-    {
-      if (priv->hadjustment)
-        g_object_set (priv->hadjustment, "page-size", 1.0f, NULL);
-      if (priv->vadjustment)
-        g_object_set (priv->vadjustment, "page-size", 1.0f, NULL);
-      return;
-    }
-
-  clutter_actor_get_clip (actor, NULL, NULL, &width, &height);
-
-  if (priv->hadjustment)
-    g_object_set (priv->hadjustment, "page-size", width, NULL);
-
-  if (priv->vadjustment)
-    g_object_set (priv->vadjustment, "page-size", height, NULL);
-}
-
 static void
 nbtk_viewport_init (NbtkViewport *self)
 {
@@ -582,10 +555,6 @@ nbtk_viewport_init (NbtkViewport *self)
 
   self->priv->sync_adjustments = TRUE;
 
-  g_signal_connect (self, "notify::clip",
-                    G_CALLBACK (clip_notify_cb), self);
-
-
   g_object_set (G_OBJECT (self), "reactive", FALSE, NULL);
 }
 
@@ -596,10 +565,10 @@ nbtk_viewport_new (void)
 }
 
 void
-nbtk_viewport_set_originu (NbtkViewport *viewport,
-                           gfloat   x,
-                           gfloat   y,
-                           gfloat   z)
+nbtk_viewport_set_origin (NbtkViewport *viewport,
+                          gfloat        x,
+                          gfloat        y,
+                          gfloat        z)
 {
   NbtkViewportPrivate *priv;
 
@@ -641,24 +610,10 @@ nbtk_viewport_set_originu (NbtkViewport *viewport,
 }
 
 void
-nbtk_viewport_set_origin (NbtkViewport *viewport,
-                          gint          x,
-                          gint          y,
-                          gint          z)
-{
-  g_return_if_fail (NBTK_IS_VIEWPORT (viewport));
-
-  nbtk_viewport_set_originu (viewport,
-                             (float) (x),
-                             (float) (y),
-                             (float) (z));
-}
-
-void
-nbtk_viewport_get_originu (NbtkViewport *viewport,
-                           gfloat  *x,
-                           gfloat  *y,
-                           gfloat  *z)
+nbtk_viewport_get_origin (NbtkViewport *viewport,
+                          gfloat       *x,
+                          gfloat       *y,
+                          gfloat       *z)
 {
   NbtkViewportPrivate *priv;
 
@@ -676,24 +631,3 @@ nbtk_viewport_get_originu (NbtkViewport *viewport,
     *z = priv->z;
 }
 
-void
-nbtk_viewport_get_origin (NbtkViewport *viewport,
-                          gint         *x,
-                          gint         *y,
-                          gint         *z)
-{
-  NbtkViewportPrivate *priv;
-
-  g_return_if_fail (NBTK_IS_VIEWPORT (viewport));
-
-  priv = viewport->priv;
-
-  if (x)
-    *x = (int) priv->x;
-
-  if (y)
-    *y = (int) priv->y;
-
-  if (z)
-    *z = (int) priv->z;
-}
