@@ -31,6 +31,7 @@
 #include <string.h>
 
 #include <clutter/clutter.h>
+#include <ccss/ccss.h>
 
 #include "nbtk-widget.h"
 
@@ -41,6 +42,8 @@
 #include "nbtk-texture-frame.h"
 #include "nbtk-tooltip.h"
 
+typedef ccss_border_image_t             NbtkBorderImage;
+
 /*
  * Forward declaration for sake of NbtkWidgetChild
  */
@@ -175,7 +178,7 @@ nbtk_widget_get_property (GObject    *gobject,
       break;
 
     case PROP_TOOLTIP_TEXT:
-      g_value_set_string (value, nbtk_tooltip_get_label (priv->tooltip));
+      g_value_set_string (value, nbtk_widget_get_tooltip_text (actor));
       break;
 
     default:
@@ -532,8 +535,7 @@ nbtk_widget_style_changed (NbtkStylable *self)
        * Firefox lets the background-image shine thru when border-image has
        * alpha an channel, maybe that would be an option for the future. */
       texture = nbtk_texture_cache_get_texture (texture_cache,
-                                                border_image->uri,
-                                                FALSE);
+                                                border_image->uri);
 
       clutter_texture_get_base_size (CLUTTER_TEXTURE (texture),
                                      &width, &height);
@@ -558,9 +560,7 @@ nbtk_widget_style_changed (NbtkStylable *self)
   if (bg_file != NULL &&
       strcmp (bg_file, "none"))
     {
-      texture = nbtk_texture_cache_get_texture (texture_cache,
-                                                bg_file,
-                                                FALSE);
+      texture = nbtk_texture_cache_get_texture (texture_cache, bg_file);
       priv->background_image = (ClutterActor*) texture;
 
       if (priv->background_image != NULL)
@@ -1267,7 +1267,13 @@ nbtk_widget_set_tooltip_text (NbtkWidget  *widget,
 const gchar*
 nbtk_widget_get_tooltip_text (NbtkWidget *widget)
 {
+  NbtkWidgetPrivate *priv;
+
   g_return_val_if_fail (NBTK_IS_WIDGET (widget), NULL);
+  priv = widget->priv;
+
+  if (!priv->has_tooltip)
+    return NULL;
 
   return nbtk_tooltip_get_label (widget->priv->tooltip);
 }
